! ========================================================================
! Copyright (C) by VMware, Inc. 1991-2011.  All Rights Reserved
!
! $Id: doublebytestring.gs 27259 2011-11-09 22:12:54Z stever $
!
! ========================================================================

removeallmethods DoubleByteString
removeallclassmethods DoubleByteString

category: 'For Documentation Installation only'
classmethod: DoubleByteString
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'A DoubleByteString is a string for which each Character occupies two bytes.

 Most of the functionality of this class is inherited from the 
 superclass MultiByteString.  See the documentation there for more details'.
doc documentClassWith: txt.

self description: doc.
%

! ------------------- Class methods for DoubleByteString

category 'Formatting'
classmethod: DoubleByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 2
%

category 'Instance Creation'
classmethod: DoubleByteString
new

"(R) Returns a new instance of the receiver."

^ self basicNew
%

category 'Instance Creation'
classmethod: DoubleByteString
new: aSize

"(R) Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of two bytes each."

^ self basicNew: (aSize * (self charSize))
%

! ------------------- Instance methods for DoubleByteString
category 'Formatting'
method: DoubleByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 2
%

category: 'Converting'
method: DoubleByteString
asDoubleByteSymbol

"Returns a symbol representation of the receiver."

^ DoubleByteSymbol withAll: self.
%

category: 'Searching'
method: DoubleByteString
speciesForCollect

"(R) Returns a class, an instance of which should be used as the result
 of collect: or other projections applied to the receiver."

^ DoubleByteString
%

category: 'Other Comparisons'
method: DoubleByteString
collatingTable

"Return the appropriate collating table for this class"

^ DoubleByteAsciiCollatingTable
%

category: 'New Indexing Comparison'
method: DoubleByteString
_idxForCompareEqualTo: arg

""

^arg _idxForCompareEqualToDoubleByteString: self
%

category: 'Accessing'
method: DoubleByteString
at: anIndex

"Returns the Character at anIndex."

<primitive: 435>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: .
self _uncontinuableError
%
category: 'Updating'
method: DoubleByteString
at: anIndex put: aChar

"(R) Stores aChar at the specified location and returns aChar."

<primitive: 436>
(aChar _class == Character) ifTrue:[
  aChar asciiValue > 16rFFFF ifTrue:[
    ^ self _convertToQuadByte at:anIndex put:aChar
  ].
] ifFalse:[
  aChar _validateClass: AbstractCharacter .
  ^ self at: anIndex put: aChar asCharacter
].
(anIndex _isInteger) ifTrue: [
  ((anIndex > (self size + 1)) _or: [anIndex <= 0]) ifTrue: [
    ^ self _errorIndexOutOfRange: anIndex
  ].
] ifFalse:[
  ^ self _errorNonIntegerIndex: anIndex
] .
self _primitiveFailed: #at:put:
%
! fix 41693
category: 'Updating'
method: DoubleByteString
size: anInteger

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

^ super _basicSize: anInteger + anInteger 
%

category: 'Private'
method:
_basicSize: anInteger

"Disallowed"
self shouldNotImplement: #_basicSize:
%
method:
_basicAt: anInteger put: aValue

"Disallowed"
self shouldNotImplement: #_basicAt:put:
%


